@@ -1,7 +1,26 @@
 package org.apache.maven.it;
 
-import junit.framework.Assert;
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
+
 import org.apache.maven.it.util.FileUtils;
+import org.apache.maven.it.util.IOUtil;
 import org.apache.maven.it.util.StringUtils;
 import org.apache.maven.it.util.cli.CommandLineException;
 import org.apache.maven.it.util.cli.CommandLineUtils;
@@ -13,23 +32,27 @@ import org.xml.sax.SAXException;
 import org.xml.sax.SAXParseException;
 import org.xml.sax.helpers.DefaultHandler;
 
-import javax.xml.parsers.ParserConfigurationException;
-import javax.xml.parsers.SAXParser;
-import javax.xml.parsers.SAXParserFactory;
 import java.io.BufferedReader;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
 import java.io.FileReader;
 import java.io.FileWriter;
 import java.io.FilenameFilter;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.InputStreamReader;
 import java.io.PrintStream;
 import java.io.Writer;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.net.URLClassLoader;
+import java.security.AccessControlException;
+import java.security.Permission;
 import java.text.DecimalFormat;
 import java.text.NumberFormat;
 import java.util.ArrayList;
@@ -40,11 +63,18 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.StringTokenizer;
+import java.util.regex.Pattern;
+
+import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.parsers.SAXParser;
+import javax.xml.parsers.SAXParserFactory;
+
+import junit.framework.Assert;
 
 /**
- * @author <a href="mailto:jason@maven.org">Jason van Zyl </a>
+ * @author Jason van Zyl
  * @author <a href="mailto:brett@apache.org">Brett Porter</a>
- * @version $Id: Verifier.java 469032 2006-10-30 00:21:48Z brett $
+ * @version $Id: Verifier.java 805828 2009-08-19 14:57:23Z bentmann $
  * @noinspection UseOfSystemOutOrSystemErr,RefusedBequest
  */
 public class Verifier
@@ -69,11 +99,22 @@ public class Verifier
 
     private Properties verifierProperties = new Properties();
 
+    private boolean autoclean = true;
+
     // TODO: needs to be configurable
     private static String localRepoLayout = "default";
 
     private boolean debug;
 
+    private boolean forkJvm = true;
+
+    private String logFileName = LOG_FILENAME;
+
+    private String defaultMavenHome;
+    
+    // will launch mvn with --debug 
+    private boolean mavenDebug = false;
+
     public Verifier( String basedir, String settingsFile )
         throws VerificationException
     {
@@ -82,11 +123,19 @@ public class Verifier
 
     public Verifier( String basedir, String settingsFile, boolean debug )
         throws VerificationException
+    {
+        this( basedir, settingsFile, debug, true );
+    }
+
+    public Verifier( String basedir, String settingsFile, boolean debug, boolean forkJvm )
+        throws VerificationException
     {
         this.basedir = basedir;
 
         this.debug = debug;
 
+        this.forkJvm = forkJvm;
+
         if ( !debug )
         {
             originalOut = System.out;
@@ -99,6 +148,26 @@ public class Verifier
         }
 
         findLocalRepo( settingsFile );
+        findDefaultMavenHome();
+    }
+
+    private void findDefaultMavenHome()
+        throws VerificationException
+    {
+        defaultMavenHome = System.getProperty( "maven.home" );
+
+        if ( defaultMavenHome == null )
+        {
+            try
+            {
+                Properties envVars = CommandLineUtils.getSystemEnvVars();
+                defaultMavenHome = envVars.getProperty( "M2_HOME" );
+            }
+            catch ( IOException e )
+            {
+                throw new VerificationException( "Cannot read system environment variables.", e );
+            }
+        }
     }
 
     public Verifier( String basedir )
@@ -113,6 +182,11 @@ public class Verifier
         this( basedir, null, debug );
     }
 
+    public void setLocalRepo( String localRepo )
+    {
+        this.localRepo = localRepo;
+    }
+
     public void resetStreams()
     {
         if ( !debug )
@@ -151,7 +225,7 @@ public class Verifier
     public void verify( boolean chokeOnErrorOutput )
         throws VerificationException
     {
-        List lines = loadFile( basedir, "expected-results.txt", false );
+        List lines = loadFile( getBasedir(), "expected-results.txt", false );
 
         for ( Iterator i = lines.iterator(); i.hasNext(); )
         {
@@ -170,33 +244,87 @@ public class Verifier
         throws VerificationException
     {
         List lines;
-        lines = loadFile( basedir, LOG_FILENAME, false );
+        lines = loadFile( getBasedir(), getLogFileName(), false );
 
         for ( Iterator i = lines.iterator(); i.hasNext(); )
         {
             String line = (String) i.next();
 
             // A hack to keep stupid velocity resource loader errors from triggering failure
-            if ( line.indexOf( "[ERROR]" ) >= 0 && line.indexOf( "VM_global_library.vm" ) == -1 )
+            if ( line.indexOf( "[ERROR]" ) >= 0 && !isVelocityError( line ) )
             {
-                throw new VerificationException( "Error in execution." );
+                throw new VerificationException( "Error in execution: " + line );
             }
         }
     }
 
+    /**
+     * Checks whether the specified line is just an error message from Velocity. Especially old versions of Doxia employ
+     * a very noisy Velocity instance.
+     * 
+     * @param line The log line to check, must not be <code>null</code>.
+     * @return <code>true</code> if the line appears to be a Velocity error, <code>false</code> otherwise.
+     */
+    private static boolean isVelocityError( String line )
+    {
+        if ( line.indexOf( "VM_global_library.vm" ) >= 0 )
+        {
+            return true;
+        }
+        if ( line.indexOf( "VM #" ) >= 0 && line.indexOf( "macro" ) >= 0 )
+        {
+            // [ERROR] VM #displayTree: error : too few arguments to macro. Wanted 2 got 0
+            return true;
+        }
+        return false;
+    }
+
+    /**
+     * Throws an exception if the text is not present in the log.
+     * @param text
+     * @throws VerificationException
+     */
+    public void verifyTextInLog( String text )
+        throws VerificationException
+    {
+        List lines;
+        lines = loadFile( getBasedir(), getLogFileName(), false );
+
+        boolean result = false;
+        for ( Iterator i = lines.iterator(); i.hasNext(); )
+        {
+            String line = (String) i.next();
+            if ( line.indexOf( text ) >= 0)
+            {
+                result = true;
+                break;
+            }
+        }
+        if (!result)
+        {
+            throw new VerificationException( "Text not found in log: " + text );
+        }
+}
+
     public Properties loadProperties( String filename )
         throws VerificationException
     {
         Properties properties = new Properties();
 
-        FileInputStream fis;
         try
         {
-            File propertiesFile = new File( basedir, filename );
+            File propertiesFile = new File( getBasedir(), filename );
             if ( propertiesFile.exists() )
             {
-                fis = new FileInputStream( propertiesFile );
-                properties.load( fis );
+                FileInputStream fis = new FileInputStream( propertiesFile );
+                try
+                {
+                    properties.load( fis );
+                }
+                finally
+                {
+                    fis.close();
+                }
             }
         }
         catch ( FileNotFoundException e )
@@ -211,6 +339,52 @@ public class Verifier
         return properties;
     }
 
+    /**
+     * Loads the (non-empty) lines of the specified text file.
+     * 
+     * @param filename The path to the text file to load, relative to the base directory, must not be <code>null</code>.
+     * @param encoding The character encoding of the file, may be <code>null</code> or empty to use the platform default
+     *            encoding.
+     * @return The list of (non-empty) lines from the text file, can be empty but never <code>null</code>.
+     * @throws IOException If the file could not be loaded.
+     * @since 1.2
+     */
+    public List loadLines( String filename, String encoding )
+        throws IOException
+    {
+        List lines = new ArrayList();
+
+        File file = new File( getBasedir(), filename );
+
+        BufferedReader reader = null;
+        try
+        {
+            if ( StringUtils.isNotEmpty( encoding ) )
+            {
+                reader = new BufferedReader( new InputStreamReader( new FileInputStream( file ), encoding ) );
+            }
+            else
+            {
+                reader = new BufferedReader( new FileReader( file ) );
+            }
+
+            String line;
+            while ( ( line = reader.readLine() ) != null )
+            {
+                if ( line.length() > 0 )
+                {
+                    lines.add( line );
+                }
+            }
+        }
+        finally
+        {
+            IOUtil.close( reader );
+        }
+
+        return lines;
+    }
+
     public List loadFile( String basedir, String filename, boolean hasCommand )
         throws VerificationException
     {
@@ -406,12 +580,62 @@ public class Verifier
         return files;
     }
 
+    /**
+     * Gets the path to the local artifact metadata. Note that the method does not check whether the returned path
+     * actually points to existing metadata.
+     * 
+     * @param gid The group id, must not be <code>null</code>.
+     * @param aid The artifact id, must not be <code>null</code>.
+     * @param version The artifact version, must not be <code>null</code>.
+     * @return The (absolute) path to the local artifact metadata, never <code>null</code>.
+     */
+    public String getArtifactMetadataPath( String gid, String aid, String version )
+    {
+        StringBuffer buffer = new StringBuffer( 256 );
+
+        buffer.append( localRepo );
+        buffer.append( '/' );
+
+        if ( "default".equals( localRepoLayout ) )
+        {
+            buffer.append( gid.replace( '.', '/' ) );
+            buffer.append( '/' );
+            buffer.append( aid );
+            buffer.append( '/' );
+            if ( version != null )
+            {
+                buffer.append( version );
+                buffer.append( '/' );
+            }
+            buffer.append( "maven-metadata-local.xml" );
+        }
+        else
+        {
+            throw new IllegalStateException( "Unsupported repository layout: " + localRepoLayout );
+        }
+
+        return buffer.toString();
+    }
+
+    /**
+     * Gets the path to the local artifact metadata. Note that the method does not check whether the returned path
+     * actually points to existing metadata.
+     * 
+     * @param gid The group id, must not be <code>null</code>.
+     * @param aid The artifact id, must not be <code>null</code>.
+     * @return The (absolute) path to the local artifact metadata, never <code>null</code>.
+     */
+    public String getArtifactMetadataPath( String gid, String aid )
+    {
+        return getArtifactMetadataPath( gid, aid, null );
+    }
+
     public void executeHook( String filename )
         throws VerificationException
     {
         try
         {
-            File f = new File( basedir, filename );
+            File f = new File( getBasedir(), filename );
 
             if ( !f.exists() )
             {
@@ -485,7 +709,7 @@ public class Verifier
         }
         else if ( "svn".equals( cmd ) )
         {
-            launchSubversion( line, basedir );
+            launchSubversion( line, getBasedir() );
         }
         else
         {
@@ -577,6 +801,121 @@ public class Verifier
         }
     }
 
+    /**
+     * Deletes all artifacts in the specified group id from the local repository.
+     * 
+     * @param gid The group id whose artifacts should be deleted, must not be <code>null</code>.
+     * @throws IOException If the artifacts could not be deleted.
+     * @since 1.2
+     */
+    public void deleteArtifacts( String gid )
+        throws IOException
+    {
+        String path;
+        if ( "default".equals( localRepoLayout ) )
+        {
+            path = gid.replace( '.', '/' );
+        }
+        else if ( "legacy".equals( localRepoLayout ) )
+        {
+            path = gid;
+        }
+        else
+        {
+            throw new IllegalStateException( "Unsupported repository layout: " + localRepoLayout );
+        }
+
+        FileUtils.deleteDirectory( new File( localRepo, path ) );
+    }
+
+    /**
+     * Deletes the specified directory.
+     * 
+     * @param path The path to the directory to delete, relative to the base directory, must not be <code>null</code>.
+     * @throws IOException If the directory could not be deleted.
+     * @since 1.2
+     */
+    public void deleteDirectory( String path )
+        throws IOException
+    {
+        FileUtils.deleteDirectory( new File( getBasedir(), path ) );
+    }
+
+    /**
+     * Writes a text file with the specified contents. The contents will be encoded using UTF-8.
+     * 
+     * @param path The path to the file, relative to the base directory, must not be <code>null</code>.
+     * @param contents The contents to write, must not be <code>null</code>.
+     * @throws IOException If the file could not be written.
+     * @since 1.2
+     */
+    public void writeFile( String path, String contents )
+        throws IOException
+    {
+        FileUtils.fileWrite( new File( getBasedir(), path ).getAbsolutePath(), "UTF-8", contents );
+    }
+
+    /**
+     * Filters a text file by replacing some user-defined tokens.
+     * 
+     * @param srcPath The path to the input file, relative to the base directory, must not be <code>null</code>.
+     * @param dstPath The path to the output file, relative to the base directory and possibly equal to the input file,
+     *            must not be <code>null</code>.
+     * @param fileEncoding The file encoding to use, may be <code>null</code> or empty to use the platform's default
+     *            encoding.
+     * @param filterProperties The mapping from tokens to replacement values, must not be <code>null</code>.
+     * @return The path to the filtered output file, never <code>null</code>.
+     * @throws IOException If the file could not be filtered.
+     * @since 1.2
+     */
+    public File filterFile( String srcPath, String dstPath, String fileEncoding, Map filterProperties )
+        throws IOException
+    {
+        File srcFile = new File( getBasedir(), srcPath );
+        String data = FileUtils.fileRead( srcFile, fileEncoding );
+
+        for ( Iterator it = filterProperties.keySet().iterator(); it.hasNext(); )
+        {
+            String token = (String) it.next();
+            String value = String.valueOf( filterProperties.get( token ) );
+            data = StringUtils.replace( data, token, value );
+        }
+
+        File dstFile = new File( getBasedir(), dstPath );
+        FileUtils.fileWrite( dstFile.getPath(), fileEncoding, data );
+
+        return dstFile;
+    }
+
+    /**
+     * Gets a new copy of the default filter properties. These default filter properties map the tokens "@basedir@" and
+     * "@baseurl@" to the test's base directory and its base <code>file:</code> URL, respectively.
+     * 
+     * @return The (modifiable) map with the default filter properties, never <code>null</code>.
+     * @since 1.2
+     */
+    public Properties newDefaultFilterProperties()
+    {
+        Properties filterProperties = new Properties();
+
+        String basedir = new File( getBasedir() ).getAbsolutePath();
+        filterProperties.put( "@basedir@", basedir );
+
+        /*
+         * NOTE: Maven fails to properly handle percent-encoded "file:" URLs (WAGON-111) so don't use File.toURI() here
+         * and just do it the simple way.
+         */
+        String baseurl = basedir;
+        if ( !baseurl.startsWith( "/" ) )
+        {
+            baseurl = '/' + baseurl;
+        }
+        baseurl = "file://" + baseurl.replace( '\\', '/' );
+        filterProperties.put( "@baseurl@", baseurl );
+
+        return filterProperties;
+    }
+
     public void assertFilePresent( String file )
     {
         try
@@ -589,6 +928,31 @@ public class Verifier
         }
     }
 
+    /**
+     * Check that given file's content matches an regular expression. Note this method also checks that the file exists
+     * and is readable.
+     *
+     * @param file the file to check.
+     * @param regex a regular expression.
+     * @see Pattern
+     */
+    public void assertFileMatches( String file, String regex )
+    {
+        assertFilePresent( file );
+        try
+        {
+            String content = FileUtils.fileRead( file );
+            if ( !Pattern.matches( regex, content ) )
+            {
+                Assert.fail( "Content of " + file + " does not match " + regex );
+            }
+        }
+        catch ( IOException e )
+        {
+            Assert.fail( e.getMessage() );
+        }
+    }
+
     public void assertFileNotPresent( String file )
     {
         try
@@ -646,7 +1010,7 @@ public class Verifier
     {
         if ( line.indexOf( "!/" ) > 0 )
         {
-            String urlString = "jar:file:" + basedir + "/" + line;
+            String urlString = "jar:file:" + getBasedir() + "/" + line;
 
             InputStream is = null;
             try
@@ -697,9 +1061,10 @@ public class Verifier
         {
             File expectedFile = new File( line );
 
-            if ( !expectedFile.isAbsolute() && !line.startsWith( "/" ) )
+            // NOTE: On Windows, a path with a leading (back-)slash is relative to the current drive
+            if ( !expectedFile.isAbsolute() && !expectedFile.getPath().startsWith( File.separator ) )
             {
-                expectedFile = new File( basedir, line );
+                expectedFile = new File( getBasedir(), line );
             }
 
             if ( line.indexOf( '*' ) > -1 )
@@ -710,7 +1075,7 @@ public class Verifier
                 {
                     if ( wanted )
                     {
-                        throw new VerificationException( "Expected file was not found: " + expectedFile.getPath() );
+                        throw new VerificationException( "Expected file pattern was not found: " + expectedFile.getPath() );
                     }
                 }
                 else
@@ -777,7 +1142,7 @@ public class Verifier
     public void executeGoal( String goal, Map envVars )
         throws VerificationException
     {
-        executeGoals( Arrays.asList( new String[]{goal} ), envVars );
+        executeGoals( Arrays.asList( new String[] { goal } ), envVars );
     }
 
     public void executeGoals( List goals )
@@ -786,11 +1151,35 @@ public class Verifier
         executeGoals( goals, Collections.EMPTY_MAP );
     }
 
+    public String getExecutable()
+    {
+        // Use a strategy for finding the maven executable, John has a simple method like this
+        // but a little strategy + chain of command would be nicer.
+
+        String mavenHome = defaultMavenHome;
+
+        if ( mavenHome != null )
+        {
+            return mavenHome + "/bin/mvn";
+        }
+        else
+        {
+            File f = new File( System.getProperty( "user.home" ), "m2/bin/mvn" );
+
+            if ( f.exists() )
+            {
+                return f.getAbsolutePath();
+            }
+            else
+            {
+                return "mvn";
+            }
+        }
+    }
+
     public void executeGoals( List goals, Map envVars )
         throws VerificationException
     {
-        String mavenHome = System.getProperty( "maven.home" );
-
         if ( goals.size() == 0 )
         {
             throw new VerificationException( "No goals specified" );
@@ -798,16 +1187,23 @@ public class Verifier
 
         List allGoals = new ArrayList();
 
-        allGoals.add( "clean:clean" );
+        if ( autoclean )
+        {
+            /*
+             * NOTE: Neither test lifecycle binding nor prefix resolution here but call the goal directly.
+             */
+            allGoals.add( "org.apache.maven.plugins:maven-clean-plugin:clean" );
+        }
 
         allGoals.addAll( goals );
 
+        Commandline cli = null;
         int ret;
 
-        File logFile = new File( basedir, LOG_FILENAME );
+        File logFile = new File( getBasedir(), getLogFileName() );
         try
         {
-            Commandline cli = new Commandline();
+            cli = createCommandLine();
 
             for ( Iterator i = envVars.keySet().iterator(); i.hasNext(); )
             {
@@ -815,48 +1211,31 @@ public class Verifier
 
                 cli.addEnvironment( key, (String) envVars.get( key ) );
 
-                try
+       /* What was the point of this? It doesn't work on windows.
+        *   try
                 {
                     FileUtils.fileWrite( "/tmp/foo.txt", "setting envar[ " + key + " = " + envVars.get( key ) );
                 }
                 catch ( IOException e )
                 {
-                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
-                }
+                    e.printStackTrace(); // To change body of catch statement use File | Settings | File Templates.
+                }*/
 
-                System.out.println();
+               // System.out.println();
             }
 
-            cli.setWorkingDirectory( basedir );
-
-            String executable;
-
-            // Use a strategy for finding the maven executable, John has a simple method like this
-            // but a little strategy + chain of command would be nicer.
-
-            if ( mavenHome != null )
+            if ( envVars.get( "JAVA_HOME" ) == null )
             {
-                executable = mavenHome + "/bin/mvn";
-            }
-            else
-            {
-                File f = new File( System.getProperty( "user.home" ), "m2/bin/mvn" );
-
-                if ( f.exists() )
-                {
-                    executable = f.getAbsolutePath();
-                }
-                else
-                {
-                    executable = "mvn";
-                }
+                cli.addEnvironment( "JAVA_HOME", System.getProperty( "java.home" ) );
             }
 
-            cli.setExecutable( executable );
+            cli.addEnvironment( "MAVEN_TERMINATE_CMD", "on" );
+
+            cli.setWorkingDirectory( getBasedir() );
 
             for ( Iterator it = cliOptions.iterator(); it.hasNext(); )
             {
-                String key = (String) it.next();
+                String key = String.valueOf( it.next() );
 
                 String resolvedArg = resolveCommandLineArg( key );
 
@@ -865,24 +1244,32 @@ public class Verifier
 
             cli.createArgument().setValue( "-e" );
 
-            cli.createArgument().setValue( "--no-plugin-registry" );
-
             cli.createArgument().setValue( "--batch-mode" );
+            
+            if ( this.mavenDebug )
+            {
+                cli.createArgument().setValue( "--debug" );
+            }
 
             for ( Iterator i = systemProperties.keySet().iterator(); i.hasNext(); )
             {
                 String key = (String) i.next();
-                cli.createArgument().setLine( "-D" + key + "=" + systemProperties.getProperty( key ) );
+                String value = systemProperties.getProperty( key );
+                cli.createArgument().setValue( "-D" + key + "=" + value );
             }
 
+            /*
+             * NOTE: Unless explicitly requested by the caller, the forked builds should use the current local
+             * repository. Otherwise, the forked builds would in principle leave the sandbox environment which has been
+             * setup for the current build. In particular, using "maven.repo.local" will make sure the forked builds use
+             * the same local repo as the parent build even if a custom user settings is provided.
+             */
             boolean useMavenRepoLocal =
                 Boolean.valueOf( verifierProperties.getProperty( "use.mavenRepoLocal", "true" ) ).booleanValue();
 
             if ( useMavenRepoLocal )
             {
-                // Note: Make sure that the repo is surrounded by quotes as it can possibly have
-                // spaces in its path.
-                cli.createArgument().setLine( "-Dmaven.repo.local=" + "\"" + localRepo + "\"" );
+                cli.createArgument().setValue( "-Dmaven.repo.local=" + localRepo );
             }
 
             for ( Iterator i = allGoals.iterator(); i.hasNext(); )
@@ -890,21 +1277,13 @@ public class Verifier
                 cli.createArgument().setValue( (String) i.next() );
             }
 
-            Writer logWriter = new FileWriter( logFile );
+           // System.out.println( "Command: " + Commandline.toString( cli.getCommandline() ) );
 
-            StreamConsumer out = new WriterStreamConsumer( logWriter );
-
-            StreamConsumer err = new WriterStreamConsumer( logWriter );
-
-            System.out.println( "Command: " + Commandline.toString( cli.getCommandline() ) );
-
-            ret = CommandLineUtils.executeCommandLine( cli, out, err );
-
-            logWriter.close();
+            ret = runCommandLine( cli, logFile );
         }
         catch ( CommandLineException e )
         {
-            throw new VerificationException( e );
+            throw new VerificationException( "Failed to execute Maven: " + cli, e );
         }
         catch ( IOException e )
         {
@@ -915,8 +1294,273 @@ public class Verifier
         {
             System.err.println( "Exit code: " + ret );
 
-            throw new VerificationException(
-                "Exit code was non-zero: " + ret + "; log = \n" + getLogContents( logFile ) );
+            throw new VerificationException( "Exit code was non-zero: " + ret + "; command line and log = \n" + cli
+                + "\n" + getLogContents( logFile ) );
+        }
+    }
+
+    public String getMavenVersion()
+        throws VerificationException
+    {
+        Commandline cmd = createCommandLine();
+        cmd.addArguments( new String[] { "--version" } );
+
+        File log;
+        try
+        {
+            log = File.createTempFile( "maven", "log" );
+        }
+        catch ( IOException e )
+        {
+            throw new VerificationException( "Error creating temp file", e );
+        }
+
+        try
+        {
+            runCommandLine( cmd, log );
+        }
+        catch ( CommandLineException e )
+        {
+            throw new VerificationException( "Error running commandline " + cmd.toString(), e );
+        }
+        catch ( IOException e )
+        {
+            throw new VerificationException( "IO Error communicating with commandline " + cmd.toString(), e );
+        }
+
+        String version = null;
+
+        List logLines = loadFile( log, false );
+        log.delete();
+
+        for ( Iterator it = logLines.iterator(); version == null && it.hasNext(); )
+        {
+            String line = (String) it.next();
+
+            final String MAVEN_VERSION = "Maven version: ";
+
+            // look out for "Maven version: 3.0-SNAPSHOT built on unknown"
+            if ( line.regionMatches( true, 0, MAVEN_VERSION, 0, MAVEN_VERSION.length() ) )
+            {
+                version = line.substring( MAVEN_VERSION.length() ).trim();
+                if ( version.indexOf( ' ' ) >= 0 )
+                {
+                    version = version.substring( 0, version.indexOf( ' ' ) );
+                }
+            }
+
+            final String NEW_MAVEN_VERSION = "Apache Maven ";
+
+            // look out for "Apache Maven 2.1.0-M2-SNAPSHOT (rXXXXXX; date)"
+            if ( line.regionMatches( true, 0, NEW_MAVEN_VERSION, 0, NEW_MAVEN_VERSION.length() ) )
+            {
+                version = line.substring( NEW_MAVEN_VERSION.length() ).trim();
+                if ( version.indexOf( ' ' ) >= 0 )
+                {
+                    version = version.substring( 0, version.indexOf( ' ' ) );
+                }
+            }
+        }
+
+        if ( version == null )
+        {
+            throw new VerificationException( "Illegal maven output: String 'Maven version: ' not found in the following output:\n"
+                + StringUtils.join( logLines.iterator(), "\n" ) );
+        }
+        else
+        {
+            return version;
+        }
+    }
+
+    private Commandline createCommandLine()
+    {
+        Commandline cmd = new Commandline();
+        String executable = getExecutable();
+        if ( executable.endsWith( "/bin/mvn" ) )
+        {
+            cmd.addEnvironment( "M2_HOME", executable.substring( 0, executable.length() - 8 ) );
+        }
+        cmd.setExecutable( executable );
+        return cmd;
+    }
+
+    private int runCommandLine( Commandline cli, File logFile )
+        throws CommandLineException, IOException
+    {
+        if ( forkJvm )
+        {
+            Writer logWriter = new FileWriter( logFile );
+
+            StreamConsumer out = new WriterStreamConsumer( logWriter );
+
+            StreamConsumer err = new WriterStreamConsumer( logWriter );
+
+            try
+            {
+                return CommandLineUtils.executeCommandLine( cli, out, err );
+            }
+            finally
+            {
+                logWriter.close();
+            }
+        }
+
+        String mavenHome = defaultMavenHome;
+
+        if ( mavenHome == null )
+        {
+            mavenHome = System.getProperty( "user.home" ) + "/local/apache-maven-2.1-SNAPSHOT";
+        }
+
+        File coreDir = new File( mavenHome, "core/boot" );
+        File[] files = coreDir.listFiles();
+        File classWorldFile = null;
+        for ( int i = 0; files != null && i < files.length; i++ )
+        {
+            if ( files[i].getName().indexOf( "plexus-classworlds" ) >= 0 )
+            {
+                classWorldFile = files[i];
+                break;
+            }
+        }
+
+        if ( classWorldFile == null )
+        {
+            throw new CommandLineException( "Cannot find plexus-classworlds in " + coreDir );
+        }
+
+        URLClassLoader cl;
+
+        try
+        {
+            cl = new URLClassLoader( new URL[] { classWorldFile.toURI().toURL() }, null );
+        }
+        catch ( MalformedURLException e )
+        {
+            throw new CommandLineException( "Cannot conver to url: " + classWorldFile, e );
+        }
+
+        class ExitSecurityException
+            extends SecurityException
+        {
+
+            private int status;
+
+            public ExitSecurityException( int status )
+            {
+                this.status = status;
+            }
+
+            public int getStatus()
+            {
+                return status;
+            }
+        }
+
+        try
+        {
+            Class c = cl.loadClass( "org.codehaus.plexus.classworlds.launcher.Launcher" );
+
+            Method m = c.getMethod( "mainWithExitCode", new Class[] { String[].class } );
+
+            SecurityManager oldSm = System.getSecurityManager();
+
+            try
+            {
+                System.setSecurityManager( new SecurityManager()
+                {
+                    public void checkPermission( Permission perm )
+                    {
+                        // ok
+                    }
+
+                    public void checkExit( int status )
+                    {
+                        throw new ExitSecurityException( status );
+                    }
+                } );
+            }
+            catch ( AccessControlException e )
+            {
+                throw new CommandLineException( "Error isntalling securitymanager", e );
+            }
+
+            cli.createArgument().setValue( "-f" );
+            cli.createArgument().setValue( cli.getWorkingDirectory().getAbsolutePath() + "/pom.xml" );
+
+            PrintStream oldOut = System.out;
+            PrintStream oldErr = System.err;
+
+            String oldCwConf = System.getProperty( "classworlds.conf" );
+            String oldMavenHome = System.getProperty( "maven.home" );
+
+            ClassLoader oldCl = Thread.currentThread().getContextClassLoader();
+
+            try
+            {
+                Thread.currentThread().setContextClassLoader( cl );// ClassLoader.getSystemClassLoader() );
+                FileOutputStream logWriter = new FileOutputStream( logFile );
+                System.setOut( new PrintStream( logWriter ) );
+                System.setErr( new PrintStream( logWriter ) );
+
+                System.setProperty( "classworlds.conf", new File( mavenHome, "bin/m2.conf" ).getAbsolutePath() );
+                System.setProperty( "maven.home", mavenHome );
+
+                return ( (Integer) m.invoke( null, new Object[] { cli.getArguments() } ) ).intValue();
+            }
+            catch ( ExitSecurityException e )
+            {
+                oldOut.println( "exit security exception caught: status=" + e.getStatus() );
+                return e.getStatus();
+            }
+            finally
+            {
+                System.setOut( oldOut );
+                System.setErr( oldErr );
+                if ( oldCwConf == null )
+                {
+                    System.getProperties().remove( "classworlds.conf" );
+                }
+                else
+                {
+                    System.setProperty( "classworlds.conf", oldCwConf );
+                }
+                if ( oldMavenHome == null )
+                {
+                    System.getProperties().remove( "maven.home" );
+                }
+                else
+                {
+                    System.setProperty( "maven.home", oldMavenHome );
+                }
+                Thread.currentThread().setContextClassLoader( oldCl );
+                System.setSecurityManager( oldSm );
+            }
+        }
+        catch ( ClassNotFoundException e )
+        {
+            throw new CommandLineException( "Cannot load classworlds launcher", e );
+        }
+        catch ( NoSuchMethodException e )
+        {
+            throw new CommandLineException( "Cannot find classworlds launcher's main method", e );
+        }
+        catch ( IllegalArgumentException e )
+        {
+            throw new CommandLineException( "Error executing classworlds launcher's main method", e );
+        }
+        catch ( InvocationTargetException e )
+        {
+            if ( e.getCause() instanceof ExitSecurityException )
+            {
+                return ( (ExitSecurityException) e.getCause() ).getStatus();
+            }
+            throw new CommandLineException( "Error executing classworlds launcher's main method", e );
+        }
+        catch ( IllegalAccessException e )
+        {
+            throw new CommandLineException( "Error executing classworlds launcher's main method", e );
         }
     }
 
@@ -935,7 +1579,7 @@ public class Verifier
 
     private String resolveCommandLineArg( String key )
     {
-        String result = key.replaceAll( "\\$\\{basedir\\}", basedir );
+        String result = key.replaceAll( "\\$\\{basedir\\}", getBasedir() );
         if ( result.indexOf( "\\\\" ) >= 0 )
         {
             result = result.replaceAll( "\\\\", "\\" );
@@ -973,7 +1617,7 @@ public class Verifier
         System.out.println( "Log file contents:" );
         try
         {
-            BufferedReader reader = new BufferedReader( new FileReader( new File( basedir, LOG_FILENAME ) ) );
+            BufferedReader reader = new BufferedReader( new FileReader( new File( getBasedir(), getLogFileName() ) ) );
             String line = reader.readLine();
             while ( line != null )
             {
@@ -1184,10 +1828,14 @@ public class Verifier
         }
 
         File repoDir = new File( localRepo );
+
         if ( !repoDir.exists() )
         {
             repoDir.mkdirs();
         }
+
+        // normalize path
+        localRepo = repoDir.getAbsolutePath();
     }
 
     private static void runIntegrationTest( Verifier verifier )
@@ -1202,9 +1850,9 @@ public class Verifier
         boolean chokeOnErrorOutput =
             Boolean.valueOf( controlProperties.getProperty( "failOnErrorOutput", "true" ) ).booleanValue();
 
-        List goals = verifier.loadFile( verifier.basedir, "goals.txt", false );
+        List goals = verifier.loadFile( verifier.getBasedir(), "goals.txt", false );
 
-        List cliOptions = verifier.loadFile( verifier.basedir, "cli-options.txt", false );
+        List cliOptions = verifier.loadFile( verifier.getBasedir(), "cli-options.txt", false );
 
         verifier.setCliOptions( cliOptions );
 
@@ -1328,8 +1976,8 @@ public class Verifier
 
         public void reset()
         {
-            this.currentBody = null;
-            this.localRepository = null;
+            currentBody = null;
+            localRepository = null;
         }
     }
 
@@ -1363,5 +2011,61 @@ public class Verifier
         this.verifierProperties = verifierProperties;
     }
 
-}
+    public boolean isAutoclean()
+    {
+        return autoclean;
+    }
 
+    public void setAutoclean( boolean autoclean )
+    {
+        this.autoclean = autoclean;
+    }
+
+    public String getBasedir()
+    {
+        return basedir;
+    }
+
+    /**
+     * Gets the name of the file used to log build output.
+     * 
+     * @return The name of the log file, relative to the base directory, never <code>null</code>.
+     * @since 1.2
+     */
+    public String getLogFileName()
+    {
+        return this.logFileName;
+    }
+
+    /**
+     * Sets the name of the file used to log build output.
+     * 
+     * @param logFileName The name of the log file, relative to the base directory, must not be empty or
+     *            <code>null</code>.
+     * @since 1.2
+     */
+    public void setLogFileName( String logFileName )
+    {
+        if ( StringUtils.isEmpty( logFileName ) )
+        {
+            throw new IllegalArgumentException( "log file name unspecified" );
+        }
+        this.logFileName = logFileName;
+    }
+
+    public void setDebug( boolean debug )
+    {
+        this.debug = debug;
+    }
+
+    public boolean isMavenDebug()
+    {
+        return mavenDebug;
+    }
+
+    public void setMavenDebug( boolean mavenDebug )
+    {
+        this.mavenDebug = mavenDebug;
+    }
+
+}
