@@ -116,7 +116,7 @@ import java.util.Vector;
  * @author <a href="mailto:Christoph.Reck@dlr.de">Christoph.Reck</a>
  * @author <a href="mailto:peter@apache.org">Peter Donald</a>
  * @author <a href="mailto:jefft@apache.org">Jeff Turner</a>
- * @version $Id: FileUtils.java 467833 2006-10-26 02:07:37Z jvanzyl $
+ * @version $Id: FileUtils.java 723852 2008-12-05 20:28:26Z bentmann $
  */
 public class FileUtils
 {
@@ -274,24 +274,37 @@ public class FileUtils
 
     public static String fileRead( File file )
         throws IOException
+    {
+        return fileRead( file, null );
+    }
+
+    public static String fileRead( File file, String encoding )
+        throws IOException
     {
         StringBuffer buf = new StringBuffer();
 
-        FileInputStream in = null;
+        Reader reader = null;
 
         try
         {
-            in = new FileInputStream( file );
-            int count;
-            byte[] b = new byte[512];
-            while ( ( count = in.read( b ) ) > 0 )  // blocking read
+            if ( encoding != null && encoding.length() > 0 )
             {
-                buf.append( new String( b, 0, count ) );
+                reader = new InputStreamReader( new FileInputStream( file ), encoding );
+            }
+            else
+            {
+                reader = new InputStreamReader( new FileInputStream( file ) );
+            }
+            int count;
+            char[] b = new char[512];
+            while ( ( count = reader.read( b ) ) > 0 ) // blocking read
+            {
+                buf.append( b, 0, count );
             }
         }
         finally
         {
-            IOUtil.close( in );
+            IOUtil.close( reader );
         }
 
         return buf.toString();
@@ -326,12 +339,32 @@ public class FileUtils
      */
     public static void fileWrite( String fileName, String data )
         throws IOException
+    {
+        fileWrite( fileName, null, data );
+    }
+
+    /**
+     * Writes data to a file. The file will be created if it does not exist.
+     * 
+     * @param fileName The name of the file to write.
+     * @param encoding The encoding of the file.
+     * @param data The content to write to the file.
+     */
+    public static void fileWrite( String fileName, String encoding, String data )
+        throws IOException
     {
         FileOutputStream out = null;
         try
         {
             out = new FileOutputStream( fileName );
-            out.write( data.getBytes() );
+            if ( encoding != null && encoding.length() > 0 )
+            {
+                out.write( data.getBytes( encoding ) );
+            }
+            else
+            {
+                out.write( data.getBytes() );
+            }
         }
         finally
         {
@@ -1148,18 +1181,18 @@ public class FileUtils
     public static void forceDelete( final File file )
         throws IOException
     {
-        if ( !file.exists() )
-        {
-            return;
-        }
-
         if ( file.isDirectory() )
         {
             deleteDirectory( file );
         }
         else
         {
-            if ( !deleteFile( file ) )
+            /*
+             * NOTE: Always try to delete the file even if it appears to be non-existent. This will ensure that a
+             * symlink whose target does not exist is deleted, too.
+             */
+            boolean filePresent = file.getCanonicalFile().exists();
+            if ( !deleteFile( file ) && filePresent )
             {
                 final String message = "File " + file + " unable to be deleted.";
                 throw new IOException( message );
@@ -1182,7 +1215,7 @@ public class FileUtils
 
         if ( !file.delete() )
         {
-            if ( System.getProperty( "os.name" ).toLowerCase().indexOf( "windows" ) > -1 )
+            if ( Os.isFamily( Os.FAMILY_WINDOWS ) )
             {
                 System.gc();
             }
@@ -1363,6 +1396,12 @@ public class FileUtils
         IOException exception = null;
 
         final File[] files = directory.listFiles();
+
+        if ( files == null )
+        {
+            return;
+        }
+
         for ( int i = 0; i < files.length; i++ )
         {
             final File file = files[i];
